// Based on http://msdn2.microsoft.com/en-us/library/system.flagsattribute.aspx
using System;
using System.Collections.Generic;
using System.Text;

namespace B_EnumsWithFlags
{
    class EnumsWithFlags
    {
        // Define an Enum without FlagsAttribute.
        enum SingleHue : short
        {   
            Black = 0,
            Red = 1,
            Green = 2,
            Blue = 4
        };

        // Define an Enum with FlagsAttribute.
        [FlagsAttribute] 
        enum MultiHue : short
        {
            Black = 0,
            Red = 1,
            Green = 2,
            Blue = 4,
            Cyan = Green | Blue
        };

        static void Main( )
        {
            Console.WriteLine("Enum without FlagsAttribute:" );
            
            // Display all possible combinations of values.
            for( int val = 0; val <= 5; val++ )
                Console.WriteLine( "{0,3} - {1}", 
                    val, ( (SingleHue)val ).ToString( ) );

            Console.WriteLine("Enum with FlagsAttribute:" );
            
            // Display all possible combinations of values.
            // Also display an invalid value.
            for( int val = 0; val <= 8; val++ )
                Console.WriteLine( "{0,3} - {1}", 
                    val, ( (MultiHue)val ).ToString( ) );
            MultiHue m=MultiHue.Black;
            // Adding Blue 
            m |= MultiHue.Blue;
            Console.WriteLine("After adding blue the value is {0}",m);
            m |= MultiHue.Green;
            Console.WriteLine("After adding green the value is {0}",m);
            if ((m & MultiHue.Blue)!=0)
            {
                Console.WriteLine("  We have the blue flag set");
            }
            else
            {
                Console.WriteLine("  The blue flag is not set");
            }
            if (m == MultiHue.Green)
            {
                Console.WriteLine("  The actual color is green");
            }
            else
            {
                Console.WriteLine("  The color is not exactly green");
            }
            m &= ~MultiHue.Blue;
            Console.WriteLine("After removing blue the value is {0}",m);
            if ((m & MultiHue.Blue)!=0)
            {
                Console.WriteLine("  We have the blue flag set");
            }
            else
            {
                Console.WriteLine("  The blue flag is not set");
            }
            if (m == MultiHue.Green)
            {
                Console.WriteLine("  The actual color is green");
            }
            else
            {
                Console.WriteLine("  The color is not exactly green");
            }

            Console.ReadLine();
        } 
    }
}
